package com.lediglesesalplass;

import java.util.ArrayList;
import java.util.Comparator;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.TextView;

@SuppressLint("DefaultLocale")
public class ClosestFragment extends Fragment{

	ReadingRoomAdapter adapter;
	ArrayList<ReadingRoom> original = Model.getAllRooms();
	@SuppressWarnings("unchecked")
	ArrayList<ReadingRoom> searchList = (ArrayList<ReadingRoom>) original.clone();
	EditText searchBox;
	ImageView deleteIcon, searchIcon;
	TextView lastUpdate;
	ListView lv;
	LocationManager locationManager;
	LocationListener locationListener;
	final String TAG = "closest";
	
	
	public void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);
		adapter = new ReadingRoomAdapter(getActivity(), R.layout.list_item, searchList);
		this.locationListener = new MyLocationListener();
		this.locationManager = (LocationManager) getActivity().getSystemService(Context.LOCATION_SERVICE);
		this.locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 10000, 0, locationListener);		
	}

	// Adding reading rooms to each row in list
	@SuppressLint("NewApi")
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		View rootView = inflater.inflate(R.layout.closest_fragment, container, false);
		lv = (ListView) rootView.findViewById(R.id.closestlist);

		// Timestamp  
		lastUpdate = (TextView) rootView.findViewById(R.id.lastUpdate);
		lastUpdate.setText("Last measurement: "+ Model.getLatestTimestamp());

		lv.setEmptyView(rootView.findViewById(R.id.empty));
		
		// Sorting based on closest rooms
		adapter.sort(new Comparator<ReadingRoom>(){
			@Override
			public int compare(ReadingRoom first, ReadingRoom second) {
				
				return getCompareByDistance(first, second);				
			}
		});
		try {
			lv.setAdapter(adapter);

		} catch (Exception e) {
			System.out.println("Setadapter i ClosestFragment feilet: "+e.getStackTrace());
		}

		//searchIcon
		searchIcon = (ImageView) rootView.findViewById(R.id.searchImage);
		searchIcon.setVisibility(View.VISIBLE); 

		//deleteICon
		deleteIcon = (ImageView) rootView.findViewById(R.id.deleteImage);
		deleteIcon.setVisibility(View.GONE); 
		deleteIcon.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				searchBox.getText().clear();
				searchIcon.setVisibility(View.VISIBLE);
				deleteIcon.setVisibility(View.GONE);
			}
		});

		//settingsButton
		final ImageView settingsButton = (ImageView) rootView.findViewById(R.id.settingsButton);
		settingsButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent intent = new Intent(getActivity(), SettingsActivity.class);
				startActivity(intent);						
			}
		});

		// Search bar
		searchBox = (EditText) rootView.findViewById(R.id.searchBox);
		searchBox.clearFocus();
		searchBox.addTextChangedListener(new TextWatcher(){

			@Override
			public void onTextChanged(CharSequence arg0, int arg1, int arg2, int arg3) {

				searchIcon.setVisibility(View.GONE);
				deleteIcon.setVisibility(View.VISIBLE);

				String searchString = searchBox.getText().toString();
				int textLength=searchString.length();
				int listSize = original.size();

				if (searchString.isEmpty()){
					searchIcon.setVisibility(View.VISIBLE);
					deleteIcon.setVisibility(View.GONE);
				}

				searchList.clear();

				for (int i = 0; i < listSize; i++) {
					String room = original.get(i).getName();
					String building = original.get(i).getBuilding();
					
					// Search in room name
					if(textLength <= room.length() ){
						// Compare the String in editText with Names in arraylist
						if(original.get(i).getName().toLowerCase().contains(searchString.toLowerCase().trim())){
							searchList.add(original.get(i));
						}
					}
					if(! searchList.contains(original.get(i)) ){						
						// Search in building name
						if(textLength <= building.length()){
							if(original.get(i).getBuilding().toLowerCase().contains(searchString.toLowerCase().trim())){
								searchList.add(original.get(i));
							}
						}

					}
				}
				// Sorting the search list
				adapter.sort(new Comparator<ReadingRoom>(){
					@Override
					public int compare(ReadingRoom first, ReadingRoom second) {
						
						return getCompareByName(first, second);				
					}
				});
				adapter.notifyDataSetChanged();
			
			}// end onTextChanged()


			@Override
			public void afterTextChanged(Editable arg0) {
				// TODO Auto-generated method stub		
			}
			@Override
			public void beforeTextChanged(CharSequence arg0, int arg1,
					int arg2, int arg3) {
				// TODO Auto-generated method stub				
			}			
		}); // end textChangedListener

		
		// Opens the room information screen for selected room
		lv.setOnItemClickListener(new OnItemClickListener() {
			
			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int position,
					long arg3) {

				// Getting the right roomId from adapter
				int id = adapter.getItem(position).getId();

				// Sending data to InformationActivity class
				Intent intent = new Intent(getActivity(), InformationActivity.class);
				intent.putExtra("roomID", id);
				startActivity(intent);

			}
		});

		return rootView;
	}

	@Override
	public void onResume(){
		super.onResume();
		updateListValues();
		lastUpdate.setText("Last measurement: "+ Model.getLatestTimestamp());
	}

	public void updateListValues(){
		this.adapter.notifyDataSetInvalidated();
		Model.updateDevices();
		original = Model.getAllRooms();
		this.adapter.sort(new Comparator<ReadingRoom>(){
			@Override
			public int compare(ReadingRoom first, ReadingRoom second) {
				
				return getCompareByDistance(first, second);				
			}
		});
		this.adapter.notifyDataSetChanged();
	}
		
	// This method sorts the list by room's filling percentage
	private int getCompareByPercent(ReadingRoom first, ReadingRoom second){
		// Compares filling percentage
		int c = Integer.valueOf(first.getPercentInt()).compareTo(Integer.valueOf(second.getPercentInt()));	
		
		// If rooms have the same filling percentage, sort by room name
		if (c == 0){
			c = first.getName().compareToIgnoreCase(second.getName());
		}		
		// If all above is equal, sort by building name 
		if(c == 0){
			c = first.getBuilding().compareToIgnoreCase(second.getBuilding());
		}
		return c;
	}

	// This method sorts the list by room name
	private int getCompareByName(ReadingRoom first, ReadingRoom second){
		
		int c = first.getName().compareToIgnoreCase(second.getName());
		if (c==0){
			c = first.getBuilding().compareToIgnoreCase(second.getBuilding());
		}
		return c;
	}

	// This method sorts the list by shortest distance from user to rooms
	private int getCompareByDistance(ReadingRoom first, ReadingRoom second){
		int c = Integer.valueOf(first.getDistance()).compareTo(Integer.valueOf(second.getDistance()));
				
		// If two rooms are at the same distance, sort by room's filling percentage
		if (c == 0){
			c = Integer.valueOf(first.getPercentInt()).compareTo(Integer.valueOf(second.getPercentInt()));
		}
		// If the rooms have the same filling percentage, sort by name 
		if (c == 0){
			c = first.getName().compareToIgnoreCase(second.getName());			
		}
		return c;
	}
	
	public static ClosestFragment newInstance(String text){

		ClosestFragment fragment = new ClosestFragment();
		Bundle b = new Bundle();
		b.putString("msg", text);

		fragment.setArguments(b);

		return fragment;
	}
	
	// Listens for updates in users' location
	private final class MyLocationListener implements LocationListener{

		@Override
		public void onLocationChanged(Location location) {
			Model.updateDistances(location);
			adapter.notifyDataSetChanged();
			adapter.sort(new Comparator<ReadingRoom>(){
				@Override
				public int compare(ReadingRoom first, ReadingRoom second) {
					
					return getCompareByDistance(first, second);				
				}
			});
		}

		@Override
		public void onProviderDisabled(String provider) {}

		@Override
		public void onProviderEnabled(String provider) {}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {}		
	}
}
