package com.lediglesesalplass;

import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class FavoriteFragment extends Fragment {

	private ListView lv;
	private ReadingRoomAdapter adapter;
	private TextView lastUpdate2;

	@Override
	public void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		View rootView = inflater.inflate(R.layout.favorite_fragment, container, false);
		lv = (ListView) rootView.findViewById(R.id.favList);

		// Timestamp
		lastUpdate2 = (TextView) rootView.findViewById(R.id.lastUpdate2);
		lastUpdate2.setText("Last measurement: "+ checkTimestamps());

		adapter = new ReadingRoomAdapter( getActivity(), R.layout.list_item, Model.getFavorites() );
		try {
			lv.setAdapter(adapter);

		} catch (Exception e) {
			System.out.println("Setadapter i FavoriteFragment feilet: "+ e.getStackTrace());
		}

		// Opens the room information screen for selected room
		lv.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int position,
					long arg3) {

				// Getting the right roomId from adapter
				ReadingRoom spot = Model.getFavorites().get(position);
				int id = adapter.getItem(position).getId();

				// Sending data to InformationActivity class
				Intent intent = new Intent(getActivity(), InformationActivity.class);
				intent.putExtra("roomID", id);

				startActivity(intent);
			}
		});
		return rootView;
	}

	// Adding readingroom to favorite list
	// addFavorite() in Model-class checks if the roomID already is added to list
	// addItems-method is called when star button is pressed
	public void addItems(int roomID){
		Model.addFavorite(roomID);
		adapter.notifyDataSetChanged();
	}

	public String checkTimestamps(){
		String latestUpdate ="00:00";
		for(int i=0; i<Model.getFavorites().size(); i++){

			if(! Model.getFavorites().get(i).getTimestamp().equals("NA")){
				if(Model.getFavorites().get(i).getTimestamp().compareTo(latestUpdate) > 0){
					latestUpdate = Model.getFavorites().get(i).getTimestamp();
				}
			}

		}
		if (latestUpdate.equals("00:00")){
			return "NA";
		}
		return latestUpdate;
	}

	public static FavoriteFragment newInstance(String text){

		FavoriteFragment fragment = new FavoriteFragment();
		Bundle b = new Bundle();
		b.putString("msg", text);

		fragment.setArguments(b);

		return fragment;
	}

	
	// Resume fragment, retrive rooms from favorite list stored in application
	@Override
	public void onResume(){
		super.onResume();
		adapter.clear();
		Model.updateDevices();
		for (ReadingRoom readingroom : Model.getFavorites()){
			adapter.add(readingroom);
		}
		adapter.notifyDataSetChanged();
		lastUpdate2.setText("Last measurement: "+ checkTimestamps());
	}
	
	@Override
	public void onPause(){
		super.onPause();
	}

}
