package com.lediglesesalplass;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.AsyncTask;
import android.util.Log;

public class GetBuildings extends AsyncTask< Void, Void, HashMap<String,String> >{

	private static final String TAG = "getbuildings"; // Used as tag for searching in logCat
	private static URI uri_mazemap;


	@Override
	protected HashMap<String, String> doInBackground(Void... params) {
		String resultString = "";
		DefaultHttpClient client = new DefaultHttpClient();

		try {
			uri_mazemap = new URI("http://use.mazemap.com/api/getallbuildings?campus=1");					

		} catch (URISyntaxException e) {
			Log.e(TAG, "URI syntax failed: "+ e.toString());
		}

		HttpGet httpGet = new HttpGet(uri_mazemap);

		try {
			HttpResponse response = client.execute(httpGet);
			StatusLine statusLine = response.getStatusLine();
			int code = statusLine.getStatusCode(); 
			if( code == 200 || code == 201 || code == 202){
				HttpEntity entity = response.getEntity();
				InputStream stream = entity.getContent();
				resultString = convertStreamToString(stream);
			} else{
				Log.e(TAG, "HTTP Statusline: " +code+" "+ statusLine.getReasonPhrase());
			}			
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		}

		// Create hashmap with <FloorID, BuildingName>
		HashMap<String, String> result = new HashMap<String, String>();
		
		try {							
			JSONArray featureArray = new JSONArray(resultString);

			//Iterate through buildings
			for(int i = 0; i < featureArray.length(); i++){
				JSONObject properties = featureArray.getJSONObject(i).getJSONObject("properties");
				JSONArray floorsArray = properties.getJSONArray("floors");
				
				// Iterate through all floors in a building and adds floorID and name in hashmap
				for (int j = 0; j < floorsArray.length(); j++){
					result.put( floorsArray.getJSONObject(j).getString("id") ,
							featureArray.getJSONObject(i).getJSONObject("properties").getString("name"));
				}
			}

		} catch (JSONException e) {
			Log.e(TAG, e.toString() );
		}
		
		return result;
	}

	// Convert httpStream to string
	private static String convertStreamToString(InputStream InStream) {
		BufferedReader BReader = new BufferedReader(new InputStreamReader(
				InStream));
		StringBuilder SBuilder = new StringBuilder();
		String TmpLine = null;

		try {
			while ((TmpLine = BReader.readLine()) != null) {
				SBuilder.append(TmpLine + "\n");
				Log.i(TAG, "Stringbuilder OK");
			}
		} catch (Exception e) {
			Log.e(TAG, e.getStackTrace().toString());
			Log.e(TAG, e.toString());

		} finally {
			try {
				InStream.close();
			} catch (Exception E) {
				E.printStackTrace();
			}
		}
		return SBuilder.toString();
	}	
}
