package com.lediglesesalplass;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.os.AsyncTask;
import android.util.Log;

@SuppressLint({ "UseSparseArrays", "SimpleDateFormat" })

public class GetDevicesAndTimestamps extends AsyncTask< Object, Void, HashMap<Integer, String[]> > {

	private static final String TAG = "getdevices"; // Used as tag for searching in logCat
	private static DefaultHttpClient httpClient = null; 
	private static HashMap<Integer, String[]> devicesAndTimestamps = new HashMap<Integer, String[]>();

	private URI uri_getDevices;
	private String stringFromServer ="";

	@Override
	protected HashMap<Integer, String[]> doInBackground(Object... params) {		
		
		int size = params.length;
		try {
			if(params[0] instanceof DefaultHttpClient){
				httpClient = (DefaultHttpClient) params[0];								
			} else{
				httpClient = new GetLogin().execute().get();
			}
			if( (size > 1) && (params[1] instanceof URI) ){
				uri_getDevices = (URI) params[1];
			}else{
				uri_getDevices = new URI("https", "analytics.mazemap.com", "/api/campus/1/regionlists/2/usages/", null);
			}
		}catch (Exception e) {
			Log.e(TAG, e.toString());
		}

		HttpGet httpGetDevices = new HttpGet(uri_getDevices);

		try {			
			// Retrieve devices and timestamps from Analytics API
			HttpResponse response2 = httpClient.execute(httpGetDevices);
			HttpEntity entity2 = response2.getEntity();
			InputStream stream2 = entity2.getContent();
			stringFromServer = convertStreamToString(stream2);
			findAllDevicesAndTimestamps(stringFromServer);
		} 
		catch (Exception e) {
			Log.e(TAG, e.toString());
		}
		return devicesAndTimestamps;
	}


	private static void findAllDevicesAndTimestamps(String param){
		JSONArray jsonArray;
		int captureRegionId =0;
		String devices = "0.0";
		long longTimestamp =0;
		String stringTimestamp="";

		try{
			JSONObject jsonObject = new JSONObject(param);
			jsonArray = jsonObject.getJSONArray("usages");
			
			//Finds timestamps and converts to right string format
			longTimestamp = jsonObject.getLong("tickTime");
			Calendar cal = Calendar.getInstance();
			cal.setTimeInMillis(longTimestamp);
			SimpleDateFormat formatter = new SimpleDateFormat("kk:mm"); //timer:minutter
			stringTimestamp = formatter.format(cal.getTime());
			
			// Retrieves device-values for all rooms and adds them to a hashmap where roomID is key
			for (int i = 0; i < jsonArray.length(); i++) {
				captureRegionId = jsonArray.getJSONObject(i).getInt("captureRegionId");
				devices = jsonArray.getJSONObject(i).getString("usage");
				String[] temp = new String[2];
				temp[0] = devices;
				temp[1] = stringTimestamp;

				devicesAndTimestamps.put(captureRegionId, temp);
			}

		}catch (Exception e){
			Log.e(TAG, e.toString());
		}


	}

	// Converts httpStream to string
	private static String convertStreamToString(InputStream InStream) {
		BufferedReader BReader = new BufferedReader(new InputStreamReader(InStream));
		StringBuilder SBuilder = new StringBuilder();
		String TmpLine = null;

		try {
			while ((TmpLine = BReader.readLine()) != null) {
				SBuilder.append(TmpLine + "\n");
				
			}
		} catch (Exception e) {
			Log.e(TAG, e.getStackTrace().toString());
			Log.e(TAG, e.toString());

		} finally {
			try {
				InStream.close();
			} catch (Exception e) {
				Log.e(TAG, e.getStackTrace().toString());
				Log.e(TAG, e.toString());
			}
		}
		return SBuilder.toString();
	}

}
