package com.lediglesesalplass;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.os.AsyncTask;
import android.util.Log;

@SuppressLint("UseSparseArrays")
public class GetRooms extends AsyncTask< Object, Void, ArrayList<ReadingRoom> > {

	private static final String TAG = "getrooms"; // Used as tag for searching in logCat
	private static DefaultHttpClient httpClient = null; 
	
	private static ArrayList<ReadingRoom> resultList = new ArrayList<ReadingRoom>();
	
	
	private static URI uri_getallRooms;	
	
	private static String allRoomsString ="";	

	@Override
	protected ArrayList<ReadingRoom> doInBackground(Object... params) {		
		
		try {
			if(params[0] instanceof DefaultHttpClient){
				httpClient = (DefaultHttpClient) params[0];			
			} else{
				httpClient = new GetLogin().execute().get();
			}
			uri_getallRooms = new URI("https://analytics.mazemap.com/api/campus/1/regionlists/2/?srid=4326");
			
		} catch (Exception e) {
			Log.e(TAG, e.toString());
		}

		HttpGet httpGet = new HttpGet(uri_getallRooms);
		
		try {
			// Retrieves static data from reading rooms
			HttpResponse httpResponse = httpClient.execute(httpGet);
			HttpEntity entity = httpResponse.getEntity();
			InputStream stream = entity.getContent();
			allRoomsString = convertStreamToString(stream);			
		} 
		catch (Exception e) {
			Log.e(TAG, e.toString());
		}		
		convertToReadingRooms(allRoomsString);

		return resultList;
	}

	private static void convertToReadingRooms(String param){
		String itemIdent = "";	// name of floor, e.g. U01
		String name = ""; // navnet of room
		int id = 0; // id of room
		int floorId = 0; // Id of floor, used to find building name
		try {
			JSONObject jsonObject = new JSONObject(param);
			JSONArray captureRegionsArray = jsonObject.getJSONArray("captureRegions");
			
			for (int i = 0; i < captureRegionsArray.length(); i++){	
				id = captureRegionsArray.getJSONObject(i).getInt("id"); // id of room, can be joined with captureRegionID
				name = captureRegionsArray.getJSONObject(i).getString("name"); // roomName								 
				String tempItemIdent = captureRegionsArray.getJSONObject(i).getString("itemIdent"); // floorname
				
				if (isNumber(tempItemIdent.charAt(0))){
					itemIdent = tempItemIdent.substring(0, 1);
				} else {
					itemIdent = tempItemIdent.substring(0, 2);
				}
				// room coordinates
				JSONArray geometriesArray = captureRegionsArray.getJSONObject(i).getJSONArray("geometries");
				floorId = geometriesArray.getJSONObject(0).getInt("floorId"); // id of floor			
				JSONObject geometryObject = geometriesArray.getJSONObject(0).getJSONObject("geometry");
				JSONArray coordinatesArray = geometryObject.getJSONArray("coordinates"); 
				double  longitude = coordinatesArray.getJSONArray(0).getJSONArray(0).getDouble(0);
				double  latitude = coordinatesArray.getJSONArray(0).getJSONArray(0).getDouble(1);
				resultList.add(new ReadingRoom(id, name, itemIdent, floorId, latitude, longitude) );
			}
		} catch (JSONException e) {
			Log.e(TAG, e.toString());
		}		
	}

	// Convert httpstream to string
	private static String convertStreamToString(InputStream InStream) {
		BufferedReader BReader = new BufferedReader(new InputStreamReader(InStream));
		StringBuilder SBuilder = new StringBuilder();
		String TmpLine = null;

		try {
			while ((TmpLine = BReader.readLine()) != null) {
				SBuilder.append(TmpLine + "\n");
				
			}
		} catch (Exception e) {
			Log.e(TAG, e.getStackTrace().toString());
			Log.e(TAG, e.toString());

		} finally {
			try {
				InStream.close();
			} catch (Exception E) {
				E.printStackTrace();
			}
		}
		return SBuilder.toString();
	}
	
	public static boolean isNumber(char num) {
		return (num >= 48 && num <= 57); // 0 - 9
	}

}
