package com.lediglesesalplass;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.os.AsyncTask;
import android.util.Log;

@SuppressLint("SimpleDateFormat")
public class GetStatistics extends AsyncTask<Object, Void, ArrayList<DeviceTimestamp>>{

	private static final String TAG = "GetStatistics";
	private static Date paramDate = null;
	private static int roomID = 0;
	private static DefaultHttpClient httpClient = null;
	private static URI uri = null;
	private  ArrayList<DeviceTimestamp> result;
	
	private static String stringFromServer;
	
	
	@Override
	protected ArrayList<DeviceTimestamp> doInBackground(Object... params) {
		
		
		int size = params.length;
		
		try {
			if( (size>0) && (params[0] instanceof DefaultHttpClient) ){
				httpClient = (DefaultHttpClient) params[0];								
			} else{
				httpClient = new GetLogin().execute().get();
			}
			if( (size > 1) && (params[1] instanceof Date) ){
				paramDate = (Date) params[1];
			}
			roomID = (Integer) params[2];
			
		}catch (Exception e) {
			Log.e(TAG, e.toString());
		}
		
		HttpGet httpGet = new HttpGet(makeURI());
	
		try {			
			HttpResponse response = httpClient.execute(httpGet);
			HttpEntity entity = response.getEntity();
			InputStream stream = entity.getContent();
			stringFromServer = convertStreamToString(stream);
			findStatistics(stringFromServer);
		} 
		catch (Exception e) {
			Log.e(TAG, e.toString());
		}		
		
		return result;
	}

	private void findStatistics(String streng){
		
		this.result = new ArrayList<DeviceTimestamp>();
		JSONArray jsonArray;		
		SimpleDateFormat formatter = new SimpleDateFormat("kk:mm"); // hours:minutes 
		
		try{
			JSONObject jsonObject = new JSONObject(streng);
			jsonArray = jsonObject.getJSONArray("usages");
			
			for (int i=0; i<jsonArray.length(); i++ ){
				Long tickTime = jsonArray.getJSONObject(i).getLong("tickTime");
				Calendar cal = Calendar.getInstance();
				cal.setTimeInMillis(tickTime);
				String stringTimestamp = formatter.format(cal.getTime());				
				Double doub = jsonArray.getJSONObject(i).getDouble("usage");
				int devices = (int)Math.ceil(doub); 
				
				result.add(new DeviceTimestamp(stringTimestamp, devices));
			}
		
		}catch(Exception e){
			Log.e(TAG, "findStatistics() feilet: "+ e.toString());
		}		
	}
	
	// This method returns a URI that retrieves devices between 07:00 - 19:15
	private static URI makeURI(){
				
		Calendar start = Calendar.getInstance();
		start.setTime(paramDate);
		start.set(Calendar.HOUR_OF_DAY, 7);
		start.set(Calendar.MINUTE, 0);
		String startMillis ="";
		startMillis = ""+start.getTimeInMillis();
		
		Calendar end = Calendar.getInstance();
		end.setTime(paramDate);
		end.set(Calendar.HOUR_OF_DAY, 19);
		end.set(Calendar.MINUTE, 15);
		String endMillis = "";
		endMillis = ""+end.getTimeInMillis();
		
		final String analytics = "https://analytics.mazemap.com/api/campus/1/regions/";
		try {
			uri = new URI(analytics + roomID + "/usagegraph/?from=" + startMillis + "&to=" + endMillis + "&points=25" );
		} catch (URISyntaxException e) {
			Log.e(TAG, "makeURI failed: " + e.toString());			
		}		
		return uri;	
	}

	
	// Converts httpstream to string
		private static String convertStreamToString(InputStream InStream) {
			BufferedReader BReader = new BufferedReader(new InputStreamReader(InStream));
			StringBuilder SBuilder = new StringBuilder();
			String TmpLine = null;

			try {
				while ((TmpLine = BReader.readLine()) != null) {
					SBuilder.append(TmpLine + "\n");
				}
			} catch (Exception e) {
				Log.e(TAG, e.getStackTrace().toString());
				Log.e(TAG, e.toString());

			} finally {
				try {
					InStream.close();
				} catch (Exception e) {
					Log.e(TAG, e.getStackTrace().toString());
					Log.e(TAG, e.toString());
				}
			}
			return SBuilder.toString();
		}
}
