package com.lediglesesalplass;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.view.KeyEvent;
import android.view.Menu;
import android.widget.Toast;

import com.lediglesesalplass.R;
import com.viewpagerindicator.TabPageIndicator;

public class MainActivity extends FragmentActivity implements OnSharedPreferenceChangeListener{

	/**
	 * The {@link android.support.v4.view.PagerAdapter} that will provide
	 * fragments for each of the sections. We use a
	 * {@link android.support.v4.app.FragmentPagerAdapter} derivative, which
	 * will keep every loaded fragment in memory. If this becomes too memory
	 * intensive, it may be best to switch to a
	 * {@link android.support.v4.app.FragmentStatePagerAdapter}.
	 */
	SectionsPagerAdapter mSectionsPagerAdapter;
	ViewPager mViewPager;
	TabPageIndicator tabIndicator;
	SharedPreferences settings;
	WifiManager wifi;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Sjekker om appen har internett. Hvis ikke s avslutter den.
		if (!Utility.isNetworkConnected(getApplication())){
			Toast.makeText(MainActivity.this, "Application requires internet", Toast.LENGTH_LONG).show();
			finish();
		}



		// Settings
		settings = PreferenceManager.getDefaultSharedPreferences(this);
		settings.registerOnSharedPreferenceChangeListener(this);

		// Create the adapter that will return a fragment for each of the two
		// primary sections of the app.
		mSectionsPagerAdapter = new SectionsPagerAdapter(getSupportFragmentManager());

		// Set up the ViewPager with the sections adapter.
		mViewPager = (ViewPager) findViewById(R.id.pager);
		mViewPager.setAdapter(mSectionsPagerAdapter);

		tabIndicator = (TabPageIndicator) findViewById(R.id.titles);
		tabIndicator.setViewPager(mViewPager);

		// Display pop-up window by first launch of the application
		boolean firstRun = getSharedPreferences("PREFERENCE", MODE_PRIVATE).getBoolean("firstRun", true);
		if (firstRun){
			new AlertDialog.Builder(this).setTitle("  IMPORTANT INFORMATION").setMessage("Number of people will not always be correct\n\nOccupied desks: \nGreen indicates: 0-60%\nYellow indicates: 61-90%\nRed indicates: 91-100%").setNeutralButton("OK", null).show();
			
			// Lagrer staten
			getSharedPreferences("PREFERENCE", MODE_PRIVATE).edit().putBoolean("firstRun", false).commit();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_MENU) {
			Intent intent = new Intent(this, SettingsActivity.class);
			startActivity(intent);
			return false;
		}
		return super.onKeyDown(keyCode, event);
	}

	/**
	 * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
	 * one of the sections/tabs/pages.
	 */
	public class SectionsPagerAdapter extends FragmentPagerAdapter {

		String[] content = new String[] {"Closest", "Favorite"};
		
		public SectionsPagerAdapter(FragmentManager fm) {
			super(fm);
		}

		@Override
		public Fragment getItem(int position) {     
			// getItem is called to instantiate the fragment for the given page.
			switch(position){        	
			case 0: return ClosestFragment.newInstance("closestFragment");
			case 1: return FavoriteFragment.newInstance("favoriteFragment");

			}
			return null;
		}

		@Override
		public int getCount() {
			// Returns the number of fragments to display.
			return content.length;
		}

		@Override
		public CharSequence getPageTitle(int position) {
			//	Locale l = Locale.getDefault();
			return content[position % content.length].toUpperCase();
		}

	}
	
	public void onResume(){
		super.onResume();		
		Model.updateDevices();
	}
	
	@Override
	public void onSharedPreferenceChanged(SharedPreferences arg0, String arg1) {		
		// Henter WiFi status
		wifi = (WifiManager) getSystemService(Context.WIFI_SERVICE);
		boolean wifiCheck = settings.getBoolean("example_checkbox", true);
		if (wifiCheck)
			wifi.setWifiEnabled(true);
		if (!wifiCheck)
			wifi.setWifiEnabled(false);		
	}
}
