package com.lediglesesalplass;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ExecutionException;

import org.apache.http.impl.client.DefaultHttpClient;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.AssetManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

@SuppressLint("UseSparseArrays")
public class Model {

	// This list contains all rooms stored as [roomID, ReadingRoom]. Shall always contain the most recent values
	private static Map<Integer, ReadingRoom> roomList = new HashMap<Integer, ReadingRoom>();

	// This list contains roomID to rooms added as favorite 
	private static ArrayList<Integer> favoriteList = new ArrayList<Integer>(); 

	// Used to maintain login state
	private static DefaultHttpClient httpClient = null;
	
	private static HashMap<String, String> buildingsFromApi = null; 
	private static ArrayList<ReadingRoom> roomsFromApi = null;
	private static HashMap<Integer, String[]> devicesAndTimestamps = null;

	public static boolean model_instantiated = false;
	private static final String TAG = "model"; 

	private static AssetManager assetMngr;
	
	private static LocationManager locationManager = null;
	
	private static Context staticContext = null;


	// Constructor
	public Model(Context context){

		if(!model_instantiated){			
			staticContext = context;
			assetMngr = staticContext.getAssets();
			Model.loadModel();
			model_instantiated = true;
		}
		if (Utility.isNetworkConnected(staticContext)){
			Model.getFavoritesFromPrefs();
		}
	}
	
	private static void loadModel(){

		if (Utility.isNetworkConnected(staticContext)){
			try {
				httpClient = new GetLogin().execute().get();
				buildingsFromApi = new GetBuildings().execute().get();
				roomsFromApi = new GetRooms().execute(httpClient).get();
			} 
			catch (Exception e) {
				Log.e(TAG, e.toString() );
			}

			// Joins rooms and buildings
			for (ReadingRoom spot : roomsFromApi) {
				spot.setBuildingName(buildingsFromApi.get(""+spot.getFloorId()) );				
				Model.addRoom(spot);
			}
			
			// Sets capacity for each room
			try {
				// File which contains roomID and capacity, stored in "asset" folder
				InputStream sb = assetMngr.open("Capacity.txt"); 
				InputStreamReader inputReader = new InputStreamReader(sb);
				BufferedReader buff = new BufferedReader(inputReader);
				String line ="";
				while((line = buff.readLine()) != null){
					
					// If empty line of comment, jump to next line
					if(line.equals("") || line.equals(" ") || line.substring(0, 1).equals("/")){
						continue;
					}					
					// If line starts with 5-integers (aka roomID)
					if( isNumeric(line.substring(0, 5)) ){
						String[] temp = line.split(";");
						int roomID = Integer.parseInt(temp[0]);
						int capacity=999;
						if (isNumeric(temp[1])){
							capacity = Integer.parseInt(temp[1]);						
						}
						if(roomList.containsKey(roomID)){
							roomList.get(roomID).setCapacity(capacity);
						}
						else{
							Log.d(TAG, "DENNE roomID FINNES IKKE: "+roomID);
						}						
						// If file contains info about floor
						if(temp.length == 3 && roomList.containsKey(roomID)){
							roomList.get(roomID).setFloor(""+ temp[2]);
						}						
					}
				}

				inputReader.close();
				sb.close();

			} catch (IOException e) {
				Log.e(TAG, "Setting av capacity feilet: "+ e.toString());
			}

		} //end networkcheck

		updateDevices();
	}
	
	// Updates timestamp and number of devices present in rooms
	public static void updateDevices(){

		if (Utility.isNetworkConnected(staticContext)){
			try {
				devicesAndTimestamps = new GetDevicesAndTimestamps().execute(httpClient).get();			
			} catch (Exception e) {
				Log.e(TAG, "Henting av devices og timestamp fra server feilet");
				Log.e(TAG, e.toString());	
				Log.e(TAG, e.getStackTrace().toString());
			}
			
			// If API returns results, update number of devices and timestamps
			if(devicesAndTimestamps.size() > 0){

				Iterator<Entry<Integer, ReadingRoom>> it = roomList.entrySet().iterator();			
				
				while(it.hasNext()){
					Map.Entry<Integer, ReadingRoom> pairs = (Map.Entry<Integer, ReadingRoom>)it.next(); 
					int roomId = pairs.getKey();
					// Sets the value of timestamp and devices if found in API
					if(devicesAndTimestamps.containsKey(roomId)){
						String temp[] = devicesAndTimestamps.get(pairs.getKey());
						double doub = Double.parseDouble(temp[0]);
						int devices = (int)Math.ceil(doub); 
						String timestamp = temp[1];

						roomList.get(roomId).setDevices(devices, timestamp);
					}					
					// Value not found in API, sets devices to 0 and timestamp to latest value
					else {					 
						roomList.get(roomId).setDevices(0,  getLatestTimestamp());
					}				
				}

			}			
			
		}else{
			Toast.makeText(staticContext, "No internet connection", Toast.LENGTH_LONG).show();
		}
	} //end updateDevices()

	public static void updateDistances(Location location){
		double deviceLatitude = location.getLatitude();
		double deviceLongitude= location.getLongitude();
		Iterator<Entry<Integer, ReadingRoom>> it = roomList.entrySet().iterator();
		while(it.hasNext()){
			Map.Entry<Integer, ReadingRoom> pairs = (Map.Entry<Integer, ReadingRoom>)it.next(); 
			int roomId = pairs.getKey();

			ReadingRoom room = pairs.getValue();
			float[] result = new float[1]; // List used to store result calculated on next line 
			Location.distanceBetween(room.getLatitude(), room.getLongitude(), deviceLatitude, deviceLongitude, result);
						
			int distance = Math.round(result[0]);
			
			roomList.get(roomId).setDistance(distance);
		}
	}
		
	// Returns a copy of the rooms as an ArrayList containg ReadingRoom objects
	public static ArrayList<ReadingRoom> getAllRooms(){		
		if (roomList != null && roomList.size() > 0){
			ArrayList<ReadingRoom> list = new ArrayList<ReadingRoom>();
			list.addAll(roomList.values());		
			return list;			
		}
		return new ArrayList<ReadingRoom>();
	}

	public static HashMap<String, ReadingRoom> getRoomsAsHashmap(){
		Iterator it = roomList.entrySet().iterator();
		HashMap<String, ReadingRoom> list = new HashMap<String, ReadingRoom>();
		while(it.hasNext()){
			Map.Entry<Integer, ReadingRoom> pairs = (Map.Entry<Integer, ReadingRoom>)it.next();
			list.put(pairs.getValue().getName(), pairs.getValue());			
		}		
		return list;
	}

	// This method takes a ReadingRoom object as parameter and inserts based on roomID
	public static void addRoom(ReadingRoom spot){
		roomList.put(spot.getId(), spot);
	}

	public static ReadingRoom getRoomByID(int roomID){
		return roomList.get(roomID);
	}

	// Is called when clicking favorite star in InformationActivity
	public static void addFavorite(int roomID) {
		boolean bool = false;
		// Checks if room already in list
		for (Integer favorite : favoriteList){
			if(favorite.equals(roomID)){
				bool = true;
				break;
			}
		}
		// If roomID not already in list, add
		if (bool == false && roomList.containsKey(roomID)){
			favoriteList.add(roomID);
			Model.getRoomByID(roomID).setFavorite();
			// Stores list to sharedprefs
			Model.saveFavoritesList(); 
		}
	}

	// Called when clicking favorite star in informationActivity
	public static void removeFavorite(int roomID){
		for(int i=0; i<favoriteList.size(); i++){
			if (favoriteList.get(i) == roomID){
				favoriteList.remove(favoriteList.get(i));		
				Model.getRoomByID(roomID).removeFavorite();
				// Stores list to sharedprefs
				Model.saveFavoritesList();  
				return;
			}
		}
	}

	// Returns all rooms in favoriteList as ReadingRoom objects
	public static ArrayList<ReadingRoom> getFavorites(){
		ArrayList<ReadingRoom> result = new ArrayList<ReadingRoom>();	

		for(int i=0; i < favoriteList.size(); i++){
			result.add( getRoomByID(favoriteList.get(i)) );
		}
		return result;
	}

	// Generates a random integer between min and max 
	public static int generateRandom(int min, int max){
		return (min + (int)(Math.random()*max)) ;
	}
 
	// Finds the last updated timestamp in roomList
	public static String getLatestTimestamp(){
		String latestUpdate ="00:00";
		ArrayList<ReadingRoom> tempList = getAllRooms();
		for(int i=0; i<tempList.size(); i++){
			if(! tempList.get(i).getTimestamp().equals("NA")){
				if(tempList.get(i).getTimestamp().compareTo(latestUpdate) > 0){
					latestUpdate = tempList.get(i).getTimestamp();
				}
			}
		}
		if (latestUpdate.equals("00:00")){
			return "NA";
		}else{
			return latestUpdate;
		}
	}

	// Checks if a string is a number
	public static boolean isNumeric(String str)	{
		for (char c : str.toCharArray()) {
			if (!Character.isDigit(c)) {
				return false;
			}
		}
		return true;
	}	
	
	// Stores favoriteList to SharedPreferences as a String
	public static void saveFavoritesList(){				
		String saveString = "";
		for (int i : favoriteList){
			saveString += i + ";";
		}		
		staticContext.getSharedPreferences("com.lediglesesalplass", staticContext.MODE_PRIVATE).edit().putString("favorites", saveString).commit();				
	}

	// Retrieves favoriteList from SharedPreferences
	public static void getFavoritesFromPrefs(){

		String favoritesString = staticContext.getSharedPreferences("com.lediglesesalplass", staticContext.MODE_PRIVATE).getString("favorites", "");

		if (favoritesString != ""){
			String[] favoritesMatrix = favoritesString.split(";");
			for (String roomId : favoritesMatrix){
				Model.addFavorite(Integer.parseInt(roomId));
			}			
		}
	}

	// This method retrieves statistics (timestamps and devices) from Analytics API. Called from InformationActivity.java
	public static ArrayList<DeviceTimestamp> getStatisticsFromAPI(Date date, int roomId) 
			throws InterruptedException, ExecutionException {		

		return (new GetStatistics().execute(httpClient, date, roomId).get());
	}

}
