package com.lediglesesalplass;

import java.text.DecimalFormat;
import java.util.ArrayList;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class ReadingRoomAdapter extends ArrayAdapter<ReadingRoom>{

	private final int[] listRowColors = new int[] {R.drawable.layer_lr_white, R.drawable.layer_lr_lightorange};
	private Context context;
	private ArrayList<ReadingRoom> readingSpotList;
	private float checkColor;

	public ReadingRoomAdapter(Context context, int resource, ArrayList<ReadingRoom> spotList) {
		super(context, resource, spotList);
		this.context = context;
		this.readingSpotList = spotList;
	}

	// Baserer seg p hver rad. Starter med posisjon 0, s gr det til det har skrevet ut alt i Listview.
	@Override
	public View getView(int position, View convertView, ViewGroup parent){
		ViewHolder holder;
		ReadingRoom room = readingSpotList.get(position);

		if (convertView == null){
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			convertView = inflater.inflate(R.layout.list_item, parent, false);

			holder = new ViewHolder();
			holder.nameText = (TextView) convertView.findViewById(R.id.readingSpotName);
			holder.buildingText = (TextView) convertView.findViewById(R.id.buildingName);
			holder.distanceText = (TextView) convertView.findViewById(R.id.distanceText);
			holder.capasityImage = (ImageView) convertView.findViewById(R.id.capacitySignal);
			holder.percentText = (TextView) convertView.findViewById(R.id.percentTest);
			convertView.setTag(holder);	
		} else {
			holder = (ViewHolder) convertView.getTag();
		}


		if (room != null){
			if (holder.nameText != null){
				// Henter og setter navn i lista basert p posisjon
				holder.nameText.setText(room.getName());
			}
			if (holder.buildingText != null){
				// Henter og setter bygningsnavn i lista basert p posisjon
				holder.buildingText.setText(room.getBuilding());
			}
			if (holder.distanceText != null){
				// Henter og setter distansen i lista basert p posisjon
				if (room.getDistance() < 1000){
					holder.distanceText.setText(room.getDistance()+"m");					
				}else{
					double temp = (double)room.getDistance();
					temp = temp/1000;
					DecimalFormat numberFormat = new DecimalFormat("0.0");
					holder.distanceText.setText(numberFormat.format(temp)+"km");
				}
			}
			if (holder.capasityImage != null){
				// Vise farger basert p antall personer vs kapasitet
				checkColor = room.getPercentInt();
				
				if(checkColor <= 0)
					holder.capasityImage.setImageResource(R.drawable.green0_75px);
				
				if (checkColor >= 1 && checkColor <= 30)
					holder.capasityImage.setImageResource(R.drawable.green25_75px);
				
				if (checkColor >= 31 && checkColor <= 60) 
					holder.capasityImage.setImageResource(R.drawable.green50_75px);
				
				if (checkColor >= 61 && checkColor <=90)
					holder.capasityImage.setImageResource(R.drawable.yellow75_75px);
				
				if (checkColor >= 91 || room.getBuilding().equals("IT-bygget"))
					holder.capasityImage.setImageResource(R.drawable.red100_75px);
			}
			if (holder.percentText != null){
				holder.percentText.setText(room.getPercentInt()+"%");
			}
		}
		
		int colorPosition = position % listRowColors.length;
		convertView.setBackgroundResource(listRowColors[colorPosition]);

		return convertView;
	}

	static class ViewHolder {
		TextView nameText;
		TextView buildingText;
		TextView distanceText;
		ImageView capasityImage;
		TextView percentText;
	}

}
